package com.ynfy.buss.article.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ynfy.buss.article.entity.Article;
import com.ynfy.buss.article.service.IArticleService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.base.controller.JeecgController;
import org.jeecg.common.system.query.QueryGenerator;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;

/**
 * @Description: article
 * @Author: jeecg-boot
 * @Date: 2023-12-23
 * @Version: V1.0
 */
@Api(tags = "article")
@RestController
@RequestMapping("/article")
@Slf4j
public class ArticleController extends JeecgController<Article, IArticleService> {
    @Autowired
    private IArticleService articleService;

    /**
     * 分页列表查询
     *
     * @param article
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    //@AutoLog(value = "article-分页列表查询")
    @ApiOperation(value = "article-分页列表查询", notes = "article-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<Article>> queryPageList(Article article,
                                                @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                HttpServletRequest req) {
        QueryWrapper<Article> queryWrapper = QueryGenerator.initQueryWrapper(article, req.getParameterMap());
        Page<Article> page = new Page<Article>(pageNo, pageSize);
        IPage<Article> pageList = articleService.page(page, queryWrapper);
        return Result.OK(pageList);
    }

    /**
     * 添加
     *
     * @param article
     * @return
     */
    @AutoLog(value = "article-添加")
    @ApiOperation(value = "article-添加", notes = "article-添加")
    //@RequiresPermissions("article:article:add")
    @PostMapping(value = "/add")
    public Result<String> add(@RequestBody Article article) {
        article.setStatus(1);
        articleService.save(article);
        return Result.OK("添加成功！");
    }

    /**
     * 编辑
     *
     * @param article
     * @return
     */
    @AutoLog(value = "article-编辑")
    @ApiOperation(value = "article-编辑", notes = "article-编辑")
    //@RequiresPermissions("article:article:edit")
    @RequestMapping(value = "/edit", method = {RequestMethod.PUT, RequestMethod.POST})
    public Result<String> edit(@RequestBody Article article) {
        articleService.updateById(article);
        return Result.OK("编辑成功!");
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "article-通过id删除")
    @ApiOperation(value = "article-通过id删除", notes = "article-通过id删除")
    //@RequiresPermissions("article:article:delete")
    @DeleteMapping(value = "/delete")
    public Result<String> delete(@RequestParam(name = "id", required = true) String id) {
        articleService.removeById(id);
        return Result.OK("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "article-批量删除")
    @ApiOperation(value = "article-批量删除", notes = "article-批量删除")
    //@RequiresPermissions("article:article:deleteBatch")
    @DeleteMapping(value = "/deleteBatch")
    public Result<String> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        this.articleService.removeByIds(Arrays.asList(ids.split(",")));
        return Result.OK("批量删除成功!");
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    //@AutoLog(value = "article-通过id查询")
    @ApiOperation(value = "article-通过id查询", notes = "article-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<Article> queryById(@RequestParam(name = "id", required = true) String id) {
        Article article = articleService.getById(id);
        if (article == null) {
            return Result.error("未找到对应数据");
        }
        return Result.OK(article);
    }

    /**
     * 导出excel
     *
     * @param request
     * @param article
     */
    //@RequiresPermissions("article:article:exportXls")
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, Article article) {
        return super.exportXls(request, article, Article.class, "article");
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    //@RequiresPermissions("article:article:importExcel")
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        return super.importExcel(request, response, Article.class);
    }

}
