package com.ynfy.buss.banner.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ynfy.buss.banner.entity.Banner;
import com.ynfy.buss.banner.service.IBannerService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.base.controller.JeecgController;
import org.jeecg.common.system.query.QueryGenerator;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;

/**
 * @Description: banner
 * @Author: jeecg-boot
 * @Date: 2023-12-17
 * @Version: V1.0
 */
@Api(tags = "banner")
@RestController
@RequestMapping("/banner")
@Slf4j
public class BannerController extends JeecgController<Banner, IBannerService> {
    @Autowired
    private IBannerService bannerService;

    /**
     * 分页列表查询
     *
     * @param banner
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    //@AutoLog(value = "banner-分页列表查询")
    @ApiOperation(value = "banner-分页列表查询", notes = "banner-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<Banner>> queryPageList(Banner banner,
                                               @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                               @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                               HttpServletRequest req) {
        QueryWrapper<Banner> queryWrapper = QueryGenerator.initQueryWrapper(banner, req.getParameterMap());
        Page<Banner> page = new Page<Banner>(pageNo, pageSize);
        IPage<Banner> pageList = bannerService.page(page, queryWrapper);
        return Result.OK(pageList);
    }

    /**
     * 添加
     *
     * @param banner
     * @return
     */
    @AutoLog(value = "banner-添加")
    @ApiOperation(value = "banner-添加", notes = "banner-添加")
    //@RequiresPermissions("banner:banner:add")
    @PostMapping(value = "/add")
    public Result<String> add(@RequestBody Banner banner) {
        bannerService.save(banner);
        return Result.OK("添加成功！");
    }

    /**
     * 编辑
     *
     * @param banner
     * @return
     */
    @AutoLog(value = "banner-编辑")
    @ApiOperation(value = "banner-编辑", notes = "banner-编辑")
    //@RequiresPermissions("banner:banner:edit")
    @RequestMapping(value = "/edit", method = {RequestMethod.PUT, RequestMethod.POST})
    public Result<String> edit(@RequestBody Banner banner) {
        bannerService.updateById(banner);
        return Result.OK("编辑成功!");
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "banner-通过id删除")
    @ApiOperation(value = "banner-通过id删除", notes = "banner-通过id删除")
    //@RequiresPermissions("banner:banner:delete")
    @DeleteMapping(value = "/delete")
    public Result<String> delete(@RequestParam(name = "id", required = true) String id) {
        bannerService.removeById(id);
        return Result.OK("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "banner-批量删除")
    @ApiOperation(value = "banner-批量删除", notes = "banner-批量删除")
    //@RequiresPermissions("banner:banner:deleteBatch")
    @DeleteMapping(value = "/deleteBatch")
    public Result<String> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        this.bannerService.removeByIds(Arrays.asList(ids.split(",")));
        return Result.OK("批量删除成功!");
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    //@AutoLog(value = "banner-通过id查询")
    @ApiOperation(value = "banner-通过id查询", notes = "banner-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<Banner> queryById(@RequestParam(name = "id", required = true) String id) {
        Banner banner = bannerService.getById(id);
        if (banner == null) {
            return Result.error("未找到对应数据");
        }
        return Result.OK(banner);
    }

    /**
     * 导出excel
     *
     * @param request
     * @param banner
     */
    //@RequiresPermissions("banner:banner:exportXls")
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, Banner banner) {
        return super.exportXls(request, banner, Banner.class, "banner");
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    //@RequiresPermissions("banner:banner:importExcel")
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        return super.importExcel(request, response, Banner.class);
    }

}
