package com.ynfy.buss.exam.userexam.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ynfy.buss.exam.userexam.entity.UserExam;
import com.ynfy.buss.exam.userexam.service.IUserExamService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.base.controller.JeecgController;
import org.jeecg.common.system.vo.LoginUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;

/**
 * @Description: 用户考试
 * @Author: jeecg-boot
 * @Date: 2023-03-05
 * @Version: V1.0
 */
@Api(tags = "用户考试")
@RestController
@RequestMapping("/userExam")
@Slf4j
public class UserExamController extends JeecgController<UserExam, IUserExamService> {

    @Autowired
    private IUserExamService userExamService;


    /**
     * 考生查询自己的考试记录
     *
     * @param userExam
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @ApiOperation(value = "用户考试-分页列表查询", notes = "用户考试-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<UserExam>> queryPageList(UserExam userExam,
                                                 @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                 @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                 HttpServletRequest req) {
        Page<UserExam> page = new Page<>(pageNo, pageSize);
        LoginUser user = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        userExam.setUserId(user.getId());
        IPage<UserExam> pageList = userExamService.selectPageList(page, userExam);
        return Result.OK(pageList);
    }

    /**
     * 阅卷查询考试记录，考试查看考情调用
     *
     * @param userExam
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @GetMapping(value = "/examRecordList")
    public Result<IPage<UserExam>> examRecordList(UserExam userExam,
                                                  @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                  @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                  HttpServletRequest req) {
        Page<UserExam> page = new Page<>(pageNo, pageSize);
        IPage<UserExam> pageList = userExamService.selectPageList(page, userExam);
        return Result.OK(pageList);
    }

    /**
     * 添加
     *
     * @param userExam
     * @return
     */
    @AutoLog(value = "用户考试-添加")
    @ApiOperation(value = "用户考试-添加", notes = "用户考试-添加")
    //@RequiresPermissions("userExam:add")
    @PostMapping(value = "/add")
    public Result<String> add(@RequestBody UserExam userExam) {
        userExamService.save(userExam);
        return Result.OK("添加成功！");
    }

    /**
     * 编辑
     *
     * @param userExam
     * @return
     */
    @AutoLog(value = "用户考试-编辑")
    @ApiOperation(value = "用户考试-编辑", notes = "用户考试-编辑")
    //@RequiresPermissions("userExam:edit")
    @RequestMapping(value = "/edit", method = {RequestMethod.PUT, RequestMethod.POST})
    public Result<String> edit(@RequestBody UserExam userExam) {
        userExamService.updateById(userExam);
        return Result.OK("编辑成功!");
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "用户考试-通过id删除")
    @ApiOperation(value = "用户考试-通过id删除", notes = "用户考试-通过id删除")
    //@RequiresPermissions("userExam:delete")
    @DeleteMapping(value = "/delete")
    public Result<String> delete(@RequestParam(name = "id", required = true) String id) {
        userExamService.removeById(id);
        return Result.OK("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "用户考试-批量删除")
    @ApiOperation(value = "用户考试-批量删除", notes = "用户考试-批量删除")
    //@RequiresPermissions("userExam:deleteBatch")
    @DeleteMapping(value = "/deleteBatch")
    public Result<String> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        this.userExamService.removeByIds(Arrays.asList(ids.split(",")));
        return Result.OK("批量删除成功!");
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    //@AutoLog(value = "用户考试-通过id查询")
    @ApiOperation(value = "用户考试-通过id查询", notes = "用户考试-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<UserExam> queryById(@RequestParam(name = "id", required = true) String id) {
        UserExam userExam = userExamService.getById(id);
        if (userExam == null) {
            return Result.error("未找到对应数据");
        }
        return Result.OK(userExam);
    }

    /**
     * 导出excel
     *
     * @param request
     * @param userExam
     */
    //@RequiresPermissions("userExam:exportXls")
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, UserExam userExam) {
        return super.exportXls(request, userExam, UserExam.class, "用户考试表");
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    //@RequiresPermissions("userExam:importExcel")
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        return super.importExcel(request, response, UserExam.class);
    }

}
